html, body { padding:0; margin:0; }

* { font-size: 14px; font-family: Arial, Helvetica, sans-serif; color: #000; }

.bbs { width: 600px; margin: auto; }
.bbs caption { padding: 20px 0; border-bottom: 5px solid #222; font-weight: bold; font-size: 18px; }

.bbs.list thead th { text-align: center; padding: 10px 0; background: #f7f7f7; border-bottom: 1px solid #e9e9e9; }
.bbs.list tbody td { text-align: center; padding: 7px 0; border-bottom: 1px solid #eee; }
.bbs.list .no { width: 80px; }
.bbs.list .author { width: 140px; }
.bbs.list .signdate { width: 160px; }
.bbs.list .title { text-align: left; }
.bbs.list .title a { text-decoration: none; color: #000; }
.bbs.list .title a:hover { text-decoration: underline; }

.bbs .buttons { text-align: center; }
.bbs .buttons a { display: inline-block; padding: 10px 20px; border: 1px solid #e9e9e9; text-decoration: none; color: #222; }
.bbs .buttons a:hover { border-color: #ccc; }

.bbs.view tbody th,
.bbs.view tbody td { padding: 10px 0; border-bottom: 1px solid #e9e9e9; }
.bbs.view tbody th { width: 100px; }
.bbs.view tfoot td { padding: 20px 0; }

.bbs.post input,
.bbs.post textarea { border: 1px solid #e0e0e0; resize: none; width: 90%; outline: none; }
.bbs.post input:focus,
.bbs.post textarea:focus { border-color: #ccc; }
.bbs.post textarea { height: 200px; }
.bbs.post .author { width: 50%; }

div.pagination { display: block; text-align: center; padding: 20px 0; }
div.pagination strong,
div.pagination a { display: inline-block; padding: 7px; border: 1px solid #e9e9e9; margin: 0 3px; text-decoration: none; color: #222; }
div.pagination strong { background: #222; color: #fff; border-color: #222; }
div.pagination a:hover { background: #888; border-color: #888; color: #fff; }